@@ -1,21 +1,19 @@
 import logging
-import operator
-import traceback
-import jsonpickle
-from copy import deepcopy
+from pathlib import Path
+from subprocess import Popen
+from subprocess import PIPE
+from subprocess import STDOUT
 import os
+import shutil
+from copy import deepcopy
+import builtins
 import sys
 import pkgutil
 import importlib.machinery
 import importlib.util
 import types
-import re
-from pathlib import Path
-from subprocess import Popen
-from subprocess import PIPE
-from subprocess import STDOUT
-import shutil
-import builtins
+import traceback
+import jsonpickle
 import inspect
 import argparse
 import requests
@@ -23,7 +21,9 @@ import yaml
 from tqdm import tqdm
 from zipfile import ZipFile
 from distutils.dir_util import mkpath
+import operator
 import ctypes
+import re
 
 ######## START CONTENT ########
 def INVALID_NAME():
@@ -61,6 +61,115 @@ class PackageError(Exception, metaclass=ActualType): pass
 class MethodPendingPopulation(Exception, metaclass=ActualType): pass
 
 
+#Self registration for use with json loading.
+#Any class that derives from SelfRegistering can be instantiated with:
+#   SelfRegistering("ClassName")
+#Based on: https://stackoverflow.com/questions/55973284/how-to-create-this-registering-factory-in-python/55973426
+class SelfRegistering(object):
+
+	def __init__(this, *args, **kwargs):
+		#ignore args.
+		super().__init__()
+
+	@classmethod
+	def GetSubclasses(cls):
+		for subclass in cls.__subclasses__():
+			# logging.info(f"Subclass dict: {subclass.__dict__}")
+			yield subclass
+			for subclass in subclass.GetSubclasses():
+				yield subclass
+
+	@classmethod
+	def GetClass(cls, classname):
+		for subclass in cls.GetSubclasses():
+			if subclass.__name__ == classname:
+				return subclass
+
+		# no subclass with matching classname found (and no default defined)
+		raise ClassNotFound(f"No known SelfRegistering class: {classname}")			
+
+	#TODO: How do we pass args to the subsequently called __init__()?
+	def __new__(cls, classname, *args, **kwargs):
+		toNew = cls.GetClass(classname)
+		logging.debug(f"Creating new {toNew.__name__}")
+
+		# Using "object" base class method avoids recursion here.
+		child = object.__new__(toNew)
+
+		#__dict__ is always blank during __new__ and only populated by __init__.
+		#This is only useful as a negative control.
+		# logging.debug(f"Created object of {child.__dict__}")
+
+		return child
+
+	@staticmethod
+	def RegisterAllClassesInDirectory(directory):
+		logging.debug(f"Loading SelfRegistering classes in {directory}")
+		logging.debug(f"Available modules: {os.listdir(directory)}")
+		for file in os.listdir(directory):
+			if (file.startswith('_') or not file.endswith('.py')):
+				continue
+
+			moduleName = file.split('.')[0]
+
+			# logging.debug(f"Attempting to registering classes in {moduleName}.")
+			loader = importlib.machinery.SourceFileLoader(moduleName, os.path.join(directory, file))
+			module = types.ModuleType(loader.name)
+			loader.exec_module(module)
+
+			# NOTE: the module is not actually imported in that it is available through sys.modules.
+			# However, this appears to be enough to get both inheritance and SelfRegistering functionality to work.
+			sys.modules[moduleName] = module #But just in case...
+			logging.debug(f"{moduleName} imported.")
+
+			#### Other Options ####
+			# __import__(module)
+			# OR
+			# for importer, module, _ in pkgutil.iter_modules([directory]):
+			#	 importer.find_module(module).exec_module(module) #fails with "AttributeError: 'str' object has no attribute '__name__'"
+			#	 importer.find_module(module).load_module(module) #Deprecated
+
+		# enable importing and inheritance for SelfRegistering classes
+		if (directory not in sys.path):
+			sys.path.append(directory)
+
+
+# A Datum is a base class for any object-oriented class structure.
+# This class is intended to be derived from and added to.
+# The members of this class are helpful labels along with the ability to invalidate a datum.
+class Datum(SelfRegistering):
+
+	# Don't worry about this.
+	# If you really want to know, look at SelfRegistering.
+	def __new__(cls, *args, **kwargs):
+		return object.__new__(cls)
+
+
+	def __init__(this, name=INVALID_NAME(), number=0):
+		# logging.debug("init Datum")
+
+		# Names are generally useful.
+		this.name = name
+
+		# Storing validity as a member makes it easy to generate bad return values (i.e. instead of checking for None) as well as manipulate class (e.g. each analysis step invalidates some class and all invalid class are discarded at the end of analysis).
+		this.valid = True
+
+	# Override this if you have your own validity checks.
+	def IsValid(this):
+		return this.valid == True
+
+
+	# Sets valid to true
+	# Override this if you have members you need to handle with care.
+	def MakeValid(this):
+		this.valid = True
+
+
+	# Sets valid to false.
+	def Invalidate(this):
+		this.valid = False
+
+
 # util is a namespace for any miscellaneous utilities.
 # You cannot create a util.
 class util:
@@ -203,274 +312,6 @@ class util:
 
 jsonpickle.handlers.registry.register(util.DotDict, util.DotDictPickler)
 
-
-#Self registration for use with json loading.
-#Any class that derives from SelfRegistering can be instantiated with:
-#   SelfRegistering("ClassName")
-#Based on: https://stackoverflow.com/questions/55973284/how-to-create-this-registering-factory-in-python/55973426
-class SelfRegistering(object):
-
-	def __init__(this, *args, **kwargs):
-		#ignore args.
-		super().__init__()
-
-	@classmethod
-	def GetSubclasses(cls):
-		for subclass in cls.__subclasses__():
-			# logging.info(f"Subclass dict: {subclass.__dict__}")
-			yield subclass
-			for subclass in subclass.GetSubclasses():
-				yield subclass
-
-	@classmethod
-	def GetClass(cls, classname):
-		for subclass in cls.GetSubclasses():
-			if subclass.__name__ == classname:
-				return subclass
-
-		# no subclass with matching classname found (and no default defined)
-		raise ClassNotFound(f"No known SelfRegistering class: {classname}")			
-
-	#TODO: How do we pass args to the subsequently called __init__()?
-	def __new__(cls, classname, *args, **kwargs):
-		toNew = cls.GetClass(classname)
-		logging.debug(f"Creating new {toNew.__name__}")
-
-		# Using "object" base class method avoids recursion here.
-		child = object.__new__(toNew)
-
-		#__dict__ is always blank during __new__ and only populated by __init__.
-		#This is only useful as a negative control.
-		# logging.debug(f"Created object of {child.__dict__}")
-
-		return child
-
-	@staticmethod
-	def RegisterAllClassesInDirectory(directory):
-		logging.debug(f"Loading SelfRegistering classes in {directory}")
-		logging.debug(f"Available modules: {os.listdir(directory)}")
-		for file in os.listdir(directory):
-			if (file.startswith('_') or not file.endswith('.py')):
-				continue
-
-			moduleName = file.split('.')[0]
-
-			# logging.debug(f"Attempting to registering classes in {moduleName}.")
-			loader = importlib.machinery.SourceFileLoader(moduleName, os.path.join(directory, file))
-			module = types.ModuleType(loader.name)
-			loader.exec_module(module)
-
-			# NOTE: the module is not actually imported in that it is available through sys.modules.
-			# However, this appears to be enough to get both inheritance and SelfRegistering functionality to work.
-			sys.modules[moduleName] = module #But just in case...
-			logging.debug(f"{moduleName} imported.")
-
-			#### Other Options ####
-			# __import__(module)
-			# OR
-			# for importer, module, _ in pkgutil.iter_modules([directory]):
-			#	 importer.find_module(module).exec_module(module) #fails with "AttributeError: 'str' object has no attribute '__name__'"
-			#	 importer.find_module(module).load_module(module) #Deprecated
-
-		# enable importing and inheritance for SelfRegistering classes
-		if (directory not in sys.path):
-			sys.path.append(directory)
-
-
-# A Datum is a base class for any object-oriented class structure.
-# This class is intended to be derived from and added to.
-# The members of this class are helpful labels along with the ability to invalidate a datum.
-class Datum(SelfRegistering):
-
-	# Don't worry about this.
-	# If you really want to know, look at SelfRegistering.
-	def __new__(cls, *args, **kwargs):
-		return object.__new__(cls)
-
-
-	def __init__(this, name=INVALID_NAME(), number=0):
-		# logging.debug("init Datum")
-
-		# Names are generally useful.
-		this.name = name
-
-		# Storing validity as a member makes it easy to generate bad return values (i.e. instead of checking for None) as well as manipulate class (e.g. each analysis step invalidates some class and all invalid class are discarded at the end of analysis).
-		this.valid = True
-
-	# Override this if you have your own validity checks.
-	def IsValid(this):
-		return this.valid == True
-
-
-	# Sets valid to true
-	# Override this if you have members you need to handle with care.
-	def MakeValid(this):
-		this.valid = True
-
-
-	# Sets valid to false.
-	def Invalidate(this):
-		this.valid = False
-
-
-# A DataContainer allows Data to be stored and worked with.
-# This class is intended to be derived from and added to.
-# Each DataContainer is comprised of multiple Data (see Datum.py for more).
-# NOTE: DataContainers are, themselves Data. Thus, you can nest your child classes however you would like.
-class DataContainer(Datum):
-
-	def __init__(this, name=INVALID_NAME()):
-		super().__init__(name)
-
-		# The data *this contains.
-		this.data = []
-
-
-	# RETURNS: an empty, invalid Datum.
-	def InvalidDatum(this):
-		ret = Datum()
-		ret.Invalidate()
-		return ret
-
-
-	# Sort things! Requires by be a valid attribute of all Data.
-	def SortData(this, by):
-		this.data.sort(key=operator.attrgetter(by))
-
-
-	# Adds a Datum to *this
-	def AddDatum(this, datum):
-		this.data.append(datum)
-
-
-	# RETURNS: a Datum with datumAttribute equal to match, an invalid Datum if none found.
-	def GetDatumBy(this, datumAttribute, match):
-		for d in this.data:
-			try: # within for loop 'cause maybe there's an issue with only 1 Datum and the rest are fine.
-				if (str(util.GetAttr(d, datumAttribute)) == str(match)):
-					return d
-			except Exception as e:
-				logging.error(f"{this.name} - {e.message}")
-				continue
-		return this.InvalidDatum()
-
-
-	# RETURNS: a Datum of the given name, an invalid Datum if none found.
-	def GetDatum(this, name):
-		return this.GetDatumBy('name', name)
-
-
-	# Removes all Data in toRem from *this.
-	# RETURNS: the Data removed
-	def RemoveData(this, toRem):
-		# logging.debug(f"Removing {toRem}")
-		this.data = [d for d in this.data if d not in toRem]
-		return toRem
-
-
-	# Removes all Data which match toRem along the given attribute
-	def RemoveDataBy(this, datumAttribute, toRem):
-		toRem = [d for d in this.data if str(util.GetAttr(d, datumAttribute)) in list(map(str, toRem))]
-		return this.RemoveData(toRem)
-
-
-	# Removes all Data in *this except toKeep.
-	# RETURNS: the Data removed
-	def KeepOnlyData(this, toKeep):
-		toRem = [d for d in this.data if d not in toKeep]
-		return this.RemoveData(toRem)
-
-
-	# Removes all Data except those that match toKeep along the given attribute
-	# RETURNS: the Data removed
-	def KeepOnlyDataBy(this, datumAttribute, toKeep):
-		# logging.debug(f"Keeping only class with a {datumAttribute} of {toKeep}")
-		# toRem = []
-		# for d in this.class:
-		#	 shouldRem = False
-		#	 for k in toKeep:
-		#		 if (str(util.GetAttr(d, datumAttribute)) == str(k)):
-		#			 logging.debug(f"found {k} in {d.__dict__}")
-		#			 shouldRem = True
-		#			 break
-		#	 if (shouldRem):
-		#		 toRem.append(d)
-		#	 else:
-		#		 logging.debug(f"{k} not found in {d.__dict__}")
-		toRem = [d for d in this.data if str(util.GetAttr(d, datumAttribute)) not in list(map(str, toKeep))]
-		return this.RemoveData(toRem)
-
-
-	# Removes all Data with the name "INVALID NAME"
-	# RETURNS: the removed Data
-	def RemoveAllUnlabeledData(this):
-		toRem = []
-		for d in this.data:
-			if (d.name =="INVALID NAME"):
-				toRem.append(d)
-		return this.RemoveData(toRem)
-
-
-	# Removes all invalid Data
-	# RETURNS: the removed Data
-	def RemoveAllInvalidData(this):
-		toRem = []
-		for d in this.data:
-			if (not d.IsValid()):
-				toRem.append(d)
-		return this.RemoveData(toRem)
-
-
-	# Removes all Data that have an attribute value relative to target.
-	# The given relation can be things like operator.le (i.e. <=)
-	#   See https://docs.python.org/3/library/operator.html for more info.
-	# If ignoreNames is specified, any Data of those names will be ignored.
-	# RETURNS: the Data removed
-	def RemoveDataRelativeToTarget(this, datumAttribute, relation, target, ignoreNames = []):
-		try:
-			toRem = []
-			for d in this.data:
-				if (ignoreNames and d.name in ignoreNames):
-					continue
-				if (relation(util.GetAttr(d, datumAttribute), target)):
-					toRem.append(d)
-			return this.RemoveData(toRem)
-		except Exception as e:
-			logging.error(f"{this.name} - {e.message}")
-			return []
-
-
-	# Removes any Data that have the same datumAttribute as a previous Datum, keeping only the first.
-	# RETURNS: The Data removed
-	def RemoveDuplicateDataOf(this, datumAttribute):
-		toRem = [] # list of Data
-		alreadyProcessed = [] # list of strings, not whatever datumAttribute is.
-		for d1 in this.data:
-			skip = False
-			for dp in alreadyProcessed:
-				if (str(util.GetAttr(d1, datumAttribute)) == dp):
-					skip = True
-					break
-			if (skip):
-				continue
-			for d2 in this.data:
-				if (d1 is not d2 and str(util.GetAttr(d1, datumAttribute)) == str(util.GetAttr(d2, datumAttribute))):
-					logging.info(f"Removing duplicate Datum {d2} with unique id {util.GetAttr(d2, datumAttribute)}")
-					toRem.append(d2)
-					alreadyProcessed.append(str(util.GetAttr(d1, datumAttribute)))
-		return this.RemoveData(toRem)
-
-
-	# Adds all Data from otherDataContainer to *this.
-	# If there are duplicate Data identified by the attribute preventDuplicatesOf, they are removed.
-	# RETURNS: the Data removed, if any.
-	def ImportDataFrom(this, otherDataContainer, preventDuplicatesOf=None):
-		this.data.extend(otherDataContainer.data);
-		if (preventDuplicatesOf is not None):
-			return this.RemoveDuplicateDataOf(preventDuplicatesOf)
-		return []
-
-
 # Don't import Method or Executor, even though they are required: it will cause a circular dependency.
 # Instead, pretend there's a forward declaration here and don't think too hard about it ;)
 ################################################################################
@@ -1403,157 +1244,163 @@ class StandardFunctor(Functor):
 	######## END: UTILITIES ########
 
 
-# Use an ErrorStringParser for each "parsers" in order to avoid having to override the GetObjectFromError method and create a new class for every error you want to handle.
-# ErrorStringParsers enable ErrorResolutions to be created on a per-functionality, rather than per-error basis, reducing the total amount of duplicate code.
-# Each error has a different string. In order to get the object of the error, we have to know where the object starts and ends.
-# NOTE: this assumes only 1 object per string. Maybe fancier parsing logic can be added in the future.
-#
-# startPosition is always positive
-# endPosition is always negative
-class ErrorStringParser:
-
-	def __init__(this, applicableError, startPosition, endPosition):
-		this.applicableError = applicableError
-		this.startPosition = startPosition
-		this.endPosition = endPosition
-
-	def Parse(this, errorString):
-		end = this.endPosition
-		if (not end):
-			end = len(errorString)
-		return errorString[this.startPosition:end]
-
-
-# ErrorResolution is a Functor which can be executed when an Exception is raised.
-# The goal of this class is to do some kind of work that will fix the problem on the second try of whatever generated the error.
-class ErrorResolution(StandardFunctor):
+# A DataContainer allows Data to be stored and worked with.
+# This class is intended to be derived from and added to.
+# Each DataContainer is comprised of multiple Data (see Datum.py for more).
+# NOTE: DataContainers are, themselves Data. Thus, you can nest your child classes however you would like.
+class DataContainer(Datum):
 
 	def __init__(this, name=INVALID_NAME()):
 		super().__init__(name)
 
-		# What errors, as ErrorStringParser objects, is *this prepared to handle?
-		this.parsers = []
-
-		this.error = None
-		this.errorType = ""
-		this.errorString = ""
-		this.errorObject = ""
-		this.errorResolutionStack = {}
-
-		# Provided directly from the recoverable decorator.
-		this.optionalKWArgs["obj"] = None
-		this.optionalKWArgs["function"] = None
-
-		# We do want to know whether or not we should attempt to run whatever failed again.
-		# So, let's store that in functionSucceeded. Meaning if this.functionSucceeded, try the original method again.
-		# No rollback, by default and definitely don't throw Exceptions.
-		this.enableRollback = False
-		this.functionSucceeded = True
-		this.raiseExceptions = False
-
-		this.errorShouldBeResolved = False
+		# The data *this contains.
+		this.data = []
 
 
-
-	# Put your logic here!
-	def Resolve(this):
-		# You get the following members:
-		# this.error (an Exception)
-		# this.errorString (a string cast of the Exception)
-		# this.errorType (a string)
-		# this.errorObjet (a string or whatever you return from GetObjectFromError())
-
-		# You get the following guarantees:
-		# *this has not been called on this particular error before.
-		# the error given is applicable to *this per this.parsers
-
-		###############################################
-		# Please throw errors if something goes wrong #
-		# Otherwise, set this.errorShouldBeResolved   #
-		###############################################
-		
-		pass
+	# RETURNS: an empty, invalid Datum.
+	def InvalidDatum(this):
+		ret = Datum()
+		ret.Invalidate()
+		return ret
 
 
-
-	# Helper method for creating ErrorStringParsers
-	# To use this, simply take an example output and replace the object you want to extract with "OBJECT"
-	def ApplyTo(this, error, exampleString):
-		match = re.search('OBJECT', exampleString)
-		this.parsers.append(ErrorStringParser(error, match.start(), match.end() - len(exampleString)))
+	# Sort things! Requires by be a valid attribute of all Data.
+	def SortData(this, by):
+		this.data.sort(key=operator.attrgetter(by))
 
 
-	# Get the type of this.error as a string.
-	def GetErrorType(this):
-		return type(this.error).__name__
+	# Adds a Datum to *this
+	def AddDatum(this, datum):
+		this.data.append(datum)
 
 
-	# Get an actionable object from the error.
-	# For example, if the error is 'ModuleNotFoundError', what is the module?
-	def GetObjectFromError(this):
-		for parser in this.parsers:
-			if (parser.applicableError != this.errorType):
+	# RETURNS: a Datum with datumAttribute equal to match, an invalid Datum if none found.
+	def GetDatumBy(this, datumAttribute, match):
+		for d in this.data:
+			try: # within for loop 'cause maybe there's an issue with only 1 Datum and the rest are fine.
+				if (str(util.GetAttr(d, datumAttribute)) == str(match)):
+					return d
+			except Exception as e:
+				logging.error(f"{this.name} - {e.message}")
 				continue
-
-			this.errorObject = parser.Parse(this.errorString)
-			return
-
-		raise ErrorResolutionError(f"{this.name} cannot parse error object from ({this.errorType}): {str(this.error)}.")
+		return this.InvalidDatum()
 
 
-	# Determine if this resolution method is applicable.
-	def CanProcess(this):
-		return this.GetErrorType() in [parser.applicableError for parser in this.parsers]
+	# RETURNS: a Datum of the given name, an invalid Datum if none found.
+	def GetDatum(this, name):
+		return this.GetDatumBy('name', name)
 
 
-	# Grab any known and necessary args from this.kwargs before any Fetch calls are made.
-	def ParseInitialArgs(this):
-		super().ParseInitialArgs()
-		if ('error' in this.kwargs):
-			this.error = this.kwargs.pop('error')
-			# Just assume the error is an actual Exception object.
-		else:
-			raise ErrorResolutionError(f"{this.name} was not given an error to resolve.")
-
-		this.errorString = str(this.error)
-		this.errorType = this.GetErrorType()
-
-		# Internal member to avoid processing duplicates
-		this.errorResolutionStack = this.executor.errorResolutionStack
+	# Removes all Data in toRem from *this.
+	# RETURNS: the Data removed
+	def RemoveData(this, toRem):
+		# logging.debug(f"Removing {toRem}")
+		this.data = [d for d in this.data if d not in toRem]
+		return toRem
 
 
-	# Error resolution is unchained.
-	def PopulateNext(this):
-		this.next = []
+	# Removes all Data which match toRem along the given attribute
+	def RemoveDataBy(this, datumAttribute, toRem):
+		toRem = [d for d in this.data if str(util.GetAttr(d, datumAttribute)) in list(map(str, toRem))]
+		return this.RemoveData(toRem)
 
 
-	# Override of Functor method.
-	# We'll keep calling this until an error is raised.
-	def Function(this):
-		this.functionSucceeded = True
-		this.errorShouldBeResolved = True
-		
-		if (not this.CanProcess()):
-			this.errorShouldBeResolved = False
-			return this.errorResolutionStack, this.errorShouldBeResolved
+	# Removes all Data in *this except toKeep.
+	# RETURNS: the Data removed
+	def KeepOnlyData(this, toKeep):
+		toRem = [d for d in this.data if d not in toKeep]
+		return this.RemoveData(toRem)
 
-		if (not this.errorString in this.errorResolutionStack.keys()):
-			this.errorResolutionStack.update({this.errorString:[]})
-		
-		if (this.name in this.errorResolutionStack[this.errorString]):
-			raise FailedErrorResolution(f"{this.name} already tried and failed to resolve {this.errorType}: {this.errorString}.")
 
-		this.GetObjectFromError()
+	# Removes all Data except those that match toKeep along the given attribute
+	# RETURNS: the Data removed
+	def KeepOnlyDataBy(this, datumAttribute, toKeep):
+		# logging.debug(f"Keeping only class with a {datumAttribute} of {toKeep}")
+		# toRem = []
+		# for d in this.class:
+		#	 shouldRem = False
+		#	 for k in toKeep:
+		#		 if (str(util.GetAttr(d, datumAttribute)) == str(k)):
+		#			 logging.debug(f"found {k} in {d.__dict__}")
+		#			 shouldRem = True
+		#			 break
+		#	 if (shouldRem):
+		#		 toRem.append(d)
+		#	 else:
+		#		 logging.debug(f"{k} not found in {d.__dict__}")
+		toRem = [d for d in this.data if str(util.GetAttr(d, datumAttribute)) not in list(map(str, toKeep))]
+		return this.RemoveData(toRem)
 
+
+	# Removes all Data with the name "INVALID NAME"
+	# RETURNS: the removed Data
+	def RemoveAllUnlabeledData(this):
+		toRem = []
+		for d in this.data:
+			if (d.name =="INVALID NAME"):
+				toRem.append(d)
+		return this.RemoveData(toRem)
+
+
+	# Removes all invalid Data
+	# RETURNS: the removed Data
+	def RemoveAllInvalidData(this):
+		toRem = []
+		for d in this.data:
+			if (not d.IsValid()):
+				toRem.append(d)
+		return this.RemoveData(toRem)
+
+
+	# Removes all Data that have an attribute value relative to target.
+	# The given relation can be things like operator.le (i.e. <=)
+	#   See https://docs.python.org/3/library/operator.html for more info.
+	# If ignoreNames is specified, any Data of those names will be ignored.
+	# RETURNS: the Data removed
+	def RemoveDataRelativeToTarget(this, datumAttribute, relation, target, ignoreNames = []):
 		try:
-			this.Resolve()
+			toRem = []
+			for d in this.data:
+				if (ignoreNames and d.name in ignoreNames):
+					continue
+				if (relation(util.GetAttr(d, datumAttribute), target)):
+					toRem.append(d)
+			return this.RemoveData(toRem)
 		except Exception as e:
-			logging.error(f"Error resolution with {this.name} failed: {e}")
-			util.LogStack()
-			this.functionSucceeded = False
-		
-		this.errorResolutionStack[this.errorString].append(this.name)
-		return this.errorResolutionStack, this.errorShouldBeResolved
+			logging.error(f"{this.name} - {e.message}")
+			return []
+
+
+	# Removes any Data that have the same datumAttribute as a previous Datum, keeping only the first.
+	# RETURNS: The Data removed
+	def RemoveDuplicateDataOf(this, datumAttribute):
+		toRem = [] # list of Data
+		alreadyProcessed = [] # list of strings, not whatever datumAttribute is.
+		for d1 in this.data:
+			skip = False
+			for dp in alreadyProcessed:
+				if (str(util.GetAttr(d1, datumAttribute)) == dp):
+					skip = True
+					break
+			if (skip):
+				continue
+			for d2 in this.data:
+				if (d1 is not d2 and str(util.GetAttr(d1, datumAttribute)) == str(util.GetAttr(d2, datumAttribute))):
+					logging.info(f"Removing duplicate Datum {d2} with unique id {util.GetAttr(d2, datumAttribute)}")
+					toRem.append(d2)
+					alreadyProcessed.append(str(util.GetAttr(d1, datumAttribute)))
+		return this.RemoveData(toRem)
+
+
+	# Adds all Data from otherDataContainer to *this.
+	# If there are duplicate Data identified by the attribute preventDuplicatesOf, they are removed.
+	# RETURNS: the Data removed, if any.
+	def ImportDataFrom(this, otherDataContainer, preventDuplicatesOf=None):
+		this.data.extend(otherDataContainer.data);
+		if (preventDuplicatesOf is not None):
+			return this.RemoveDuplicateDataOf(preventDuplicatesOf)
+		return []
+
 
 #from .Executor import Executor # don't import this, it'll be circular!
 
@@ -2203,7 +2050,8 @@ class Executor(DataContainer, Functor):
 				isSet = True
 
 		if (not isSet):
-			val, fetched = this.FetchWithout('globals', name, start=False)
+			logging.debug(f"Fetching {name}...")
+			val, fetched = this.FetchWithout(['globals', 'this'], name, start=False)
 			if (fetched):
 				value = this.EvaluateToType(val)
 				isSet = True
@@ -2289,6 +2137,159 @@ class Executor(DataContainer, Functor):
 	######## END: Fetch Locations ########
 
 
+# Use an ErrorStringParser for each "parsers" in order to avoid having to override the GetObjectFromError method and create a new class for every error you want to handle.
+# ErrorStringParsers enable ErrorResolutions to be created on a per-functionality, rather than per-error basis, reducing the total amount of duplicate code.
+# Each error has a different string. In order to get the object of the error, we have to know where the object starts and ends.
+# NOTE: this assumes only 1 object per string. Maybe fancier parsing logic can be added in the future.
+#
+# startPosition is always positive
+# endPosition is always negative
+class ErrorStringParser:
+
+	def __init__(this, applicableError, startPosition, endPosition):
+		this.applicableError = applicableError
+		this.startPosition = startPosition
+		this.endPosition = endPosition
+
+	def Parse(this, errorString):
+		end = this.endPosition
+		if (not end):
+			end = len(errorString)
+		return errorString[this.startPosition:end]
+
+
+# ErrorResolution is a Functor which can be executed when an Exception is raised.
+# The goal of this class is to do some kind of work that will fix the problem on the second try of whatever generated the error.
+class ErrorResolution(StandardFunctor):
+
+	def __init__(this, name=INVALID_NAME()):
+		super().__init__(name)
+
+		# What errors, as ErrorStringParser objects, is *this prepared to handle?
+		this.parsers = []
+
+		this.error = None
+		this.errorType = ""
+		this.errorString = ""
+		this.errorObject = ""
+		this.errorResolutionStack = {}
+
+		# Provided directly from the recoverable decorator.
+		this.optionalKWArgs["obj"] = None
+		this.optionalKWArgs["function"] = None
+
+		# We do want to know whether or not we should attempt to run whatever failed again.
+		# So, let's store that in functionSucceeded. Meaning if this.functionSucceeded, try the original method again.
+		# No rollback, by default and definitely don't throw Exceptions.
+		this.enableRollback = False
+		this.functionSucceeded = True
+		this.raiseExceptions = False
+
+		this.errorShouldBeResolved = False
+
+
+
+	# Put your logic here!
+	def Resolve(this):
+		# You get the following members:
+		# this.error (an Exception)
+		# this.errorString (a string cast of the Exception)
+		# this.errorType (a string)
+		# this.errorObjet (a string or whatever you return from GetObjectFromError())
+
+		# You get the following guarantees:
+		# *this has not been called on this particular error before.
+		# the error given is applicable to *this per this.parsers
+
+		###############################################
+		# Please throw errors if something goes wrong #
+		# Otherwise, set this.errorShouldBeResolved   #
+		###############################################
+		
+		pass
+
+
+
+	# Helper method for creating ErrorStringParsers
+	# To use this, simply take an example output and replace the object you want to extract with "OBJECT"
+	def ApplyTo(this, error, exampleString):
+		match = re.search('OBJECT', exampleString)
+		this.parsers.append(ErrorStringParser(error, match.start(), match.end() - len(exampleString)))
+
+
+	# Get the type of this.error as a string.
+	def GetErrorType(this):
+		return type(this.error).__name__
+
+
+	# Get an actionable object from the error.
+	# For example, if the error is 'ModuleNotFoundError', what is the module?
+	def GetObjectFromError(this):
+		for parser in this.parsers:
+			if (parser.applicableError != this.errorType):
+				continue
+
+			this.errorObject = parser.Parse(this.errorString)
+			return
+
+		raise ErrorResolutionError(f"{this.name} cannot parse error object from ({this.errorType}): {str(this.error)}.")
+
+
+	# Determine if this resolution method is applicable.
+	def CanProcess(this):
+		return this.GetErrorType() in [parser.applicableError for parser in this.parsers]
+
+
+	# Grab any known and necessary args from this.kwargs before any Fetch calls are made.
+	def ParseInitialArgs(this):
+		super().ParseInitialArgs()
+		if ('error' in this.kwargs):
+			this.error = this.kwargs.pop('error')
+			# Just assume the error is an actual Exception object.
+		else:
+			raise ErrorResolutionError(f"{this.name} was not given an error to resolve.")
+
+		this.errorString = str(this.error)
+		this.errorType = this.GetErrorType()
+
+		# Internal member to avoid processing duplicates
+		this.errorResolutionStack = this.executor.errorResolutionStack
+
+
+	# Error resolution is unchained.
+	def PopulateNext(this):
+		this.next = []
+
+
+	# Override of Functor method.
+	# We'll keep calling this until an error is raised.
+	def Function(this):
+		this.functionSucceeded = True
+		this.errorShouldBeResolved = True
+		
+		if (not this.CanProcess()):
+			this.errorShouldBeResolved = False
+			return this.errorResolutionStack, this.errorShouldBeResolved
+
+		if (not this.errorString in this.errorResolutionStack.keys()):
+			this.errorResolutionStack.update({this.errorString:[]})
+		
+		if (this.name in this.errorResolutionStack[this.errorString]):
+			raise FailedErrorResolution(f"{this.name} already tried and failed to resolve {this.errorType}: {this.errorString}.")
+
+		this.GetObjectFromError()
+
+		try:
+			this.Resolve()
+		except Exception as e:
+			logging.error(f"Error resolution with {this.name} failed: {e}")
+			util.LogStack()
+			this.functionSucceeded = False
+		
+		this.errorResolutionStack[this.errorString].append(this.name)
+		return this.errorResolutionStack, this.errorShouldBeResolved
+
+
 # Global Fetch() function.
 # Uses the latest registered Executor
 def Fetch(varName, default=None, fetchFrom=None, start=True, attempted=None):
